#!/usr/bin/env python
import astrotrails
import os, sys, time

def main():
  #Defining paths
  path = os.getcwd()
  sbsPath = os.path.join(path, "StackingSBS")

  #Checking for arguments
  if len(sys.argv) < 3:
    astrotrails.manual()
    sys.exit()
  #Initiating variables
  else:
    imageName = "Stacked.jpg"
    fps = 25
    videoName = "timelapseVideo.mp4"
    #Assigning arguments to variables
    for arg in sys.argv[1:]:
      if arg[:10] == "imageName=":
        imageName = arg[10:]
      elif arg[:4] == "fps=":
        fps = arg[4:]
      elif arg[:10] == "videoName=":
        videoName = arg[10:]
    #Assigning time and defining paths
    startTime = time.perf_counter()
    imagePath = os.path.join(path,sys.argv[1])
    #Changing directory to image path and getting the list of files
    os.chdir(imagePath)
    files = os.listdir()
    #Initiating image list
    imageList = []
    #Adding all jpgs to a sorted image list
    for imageFile in files:
      if imageFile[-4:] in [".JPG",".jpg"]:
        imageList.append(imageFile)
    imageList.sort()
    #Mode selection
    if sys.argv[2] == "1":
      astrotrails.stacking(imagePath, imageList, imageName, path)
      if not os.path.exists(sbsPath):
        os.mkdir(sbsPath)
      astrotrails.stackingSBS(imagePath, imageList, sbsPath)
      astrotrails.timelapseVideo(path, sbsPath, fps,videoName)
    elif sys.argv[2] == "2":
      astrotrails.stacking(imagePath, imageList, imageName, path)
    elif sys.argv[2] == "3":
      if not os.path.exists(sbsPath):
        os.mkdir(sbsPath)
      astrotrails.stackingSBS(imagePath, imageList, sbsPath)
      astrotrails.timelapseVideo(path, sbsPath, fps, videoName)
    elif sys.argv[2] == "4":
      astrotrails.timelapseVideo(path, sbsPath, fps, videoName)
    else:
      print("The mode number is not valid.\n")
      time.sleep(1)
      astrotrails.manual()
      sys.exit()
    #Assining end time and show the finishing message
    finishTime = time.perf_counter()
    print("Total processing time: %1.2f s" % (finishTime - startTime))
    sys.exit()

if __name__ == "__main__":
  main()



import sys, os, numpy, time
from progress.bar import Bar
from PIL import Image

def manual():
  print(23*"*")
  print("***"+(17*" ")+"***")
  print("***   Astrotrails   ***")
  print("***"+(17*" ")+"***")
  print(23*"*")
  print("Tool to generate startrails image and timelapse video using jpegs photographs.")
  print()
  print("usage: astrotrails <imagesPath> <mode> [imageName] [timelapseFPS] [timelapseVideoName]")
  print()
  print(10*" "+"<imagesPath> : The path containing the images to stack" )
  print()
  print(10*" "+"<mode> : Selection of one of the modes below:" )
  print(19*" "+"1. Both Stacking and Timelapse Video")
  print(19*" "+"2. Only Stacking")
  print(19*" "+"3. Only Timelapse Video with step by step stacking")
  print(19*" "+"4. Only Timelapse Video without step by step stacking")
  print()
  print(10*" "+"[imageName=] : Name of the output picture" )
  print()
  print(10*" "+"[fps=] : Frames per second of the timelapse video" )
  print()
  print(10*" "+"[videoName=] : Name of the timelapse video" )
  print()


def stacking(imagePath, pictureList, outputName, savePath):
  os.chdir(imagePath)
  width, height = Image.open(pictureList[0]).size
  stack = numpy.zeros((height, width, 3), float)

  bar = Bar("Stacking", max = len(pictureList))
  for pic in pictureList:
    processedPic = numpy.array(Image.open(pic), dtype = float)
    stack = numpy.maximum(stack, processedPic)

    bar.next()
    
  stack = numpy.array(numpy.round(stack), dtype = numpy.uint8)
  output = Image.fromarray(stack, mode = "RGB")
  os.chdir(savePath)
  output.save(outputName, "JPEG")
  bar.finish()


def stackingSBS(imagePath, pictureList, sbsPath):
  os.chdir(imagePath)
  width, height = Image.open(pictureList[0]).size
  stack = numpy.zeros((height, width, 3), float)
  zerosNumber = len(pictureList)
  bar = Bar("Stacking step by step", max = len(pictureList))
  for i, pic in enumerate(pictureList):
    os.chdir(imagePath)
    processedPic = numpy.array(Image.open(pic), dtype = float)
    stack = numpy.maximum(stack, processedPic)
    num = str(i)
    while len(num) < len(str(zerosNumber)):
      num = "0" + num
    bar.next()
    stack = numpy.array(numpy.round(stack), dtype = numpy.uint8)
    output = Image.fromarray(stack, mode = "RGB")
    os.chdir(sbsPath)
    output.save("Stacking_%s"%num, "JPEG")
  bar.finish()
  
  
def timelapseVideo(path, sbsPath, fps, outputName):
  os.chdir(sbsPath)
  print("Generating video...")
  os.system("ffmpeg -r %s -f image2 -pattern_type glob -i 'Stacking_*' -vcodec libx264 %s/%s -y >/dev/null 2>&1"%(fps, path, outputName))
  



import astrotrails
import os
import customtkinter
from customtkinter import filedialog
import threading
import ttkthemes
from PIL import Image


customtkinter.set_appearance_mode("dark")
customtkinter.set_default_color_theme("dark-blue")

class App(customtkinter.CTk):
    def __init__(self):
        super().__init__()
        self.title("Astrotrails")
        self.geometry("700x450")
        self.style = ttkthemes.ThemedStyle()
        self.style.theme_use('breeze')
        frame=MainFrame(self)
        self.mainloop()


class MainFrame(customtkinter.CTkFrame):
    def __init__(self,master):
        super().__init__(master)
        self.createWidgets()
        self.pack(fill="both",expand=True)

    def createWidgets(self):
        label = customtkinter.CTkLabel(master=self, text="Astrotrails", font=("calibri",24))
        label.pack(pady=50)
        getDir = customtkinter.CTkButton(master=self, text="Open Directory", font=("calibri",20), command=self.getDirectory)
        getDir.pack(side="left", padx=20, expand=True)
        gen = customtkinter.CTkButton(master=self, text="Generate", font=("calibri",20), command=self.generate)
        gen.pack(side="right", padx=20, expand=True)
        global mode
        mode = customtkinter.CTkComboBox(master=self, width=220, values=["Stacking & Video", "Stacking", "Video"], font=("calibri",20))
        mode.pack( padx=20, expand=True)
        mode.set("Stacking & Video")

    def getDirectory(self):
        global directory
        directory = filedialog.askdirectory(initialdir="~")

    def generate(self):
        global sbsPath, imageList, selectedMode, imageName, fps, videoName, path
        path = os.path.dirname(directory)
        imageName = "Stacked.jpg"
        fps = 25
        videoName = "timelapseVideo.mp4"
        sbsPath = os.path.join(directory, "StackingSBS")
        os.chdir(directory)
        files = os.listdir()
        #Initiating image list
        imageList = []
        #Adding all jpgs to a sorted image list
        for imageFile in files:
            if imageFile[-4:] in [".JPG",".jpg"]:
                imageList.append(imageFile)
        imageList.sort()
        selectedMode = mode.get()
        g = generation(self)


class generation(customtkinter.CTkToplevel):
    def __init__(self,master):
        super().__init__(master)
        self.title("Astrotrails generation")
        self.geometry("400x200")
        self.progress()

    def progress(self):
        global pb, generationText, infoText, infoLabel
        generationText =customtkinter.StringVar()
        generationText.set("Generating startrails...")
        generationLabel = customtkinter.CTkLabel(master=self, textvariable=generationText, font=("calibri",24))
        generationLabel.pack(pady=20)
        infoText = customtkinter.StringVar()
        infoLabel = customtkinter.CTkLabel(master=self, textvariable=infoText, font=("calibri",16))
        infoLabel.pack(pady=20)
        pb = customtkinter.CTkProgressBar(master=self)
        pb.pack()
        threading.Thread(target=self.astro).start()

    def astro(self):
        pb.start()
        if selectedMode == "Stacking & Video":
            infoText.set("Current process: Stacking")
            astrotrails.stacking(directory, imageList, imageName, path)
            if not os.path.exists(sbsPath):
                os.mkdir(sbsPath)
            infoText.set("Current process: Stacking Step by step")
            astrotrails.stackingSBS(directory, imageList, sbsPath)
            infoText.set("Current process: Video generation")
            astrotrails.timelapseVideo(path, sbsPath, fps, videoName)
        elif selectedMode == "Stacking":
            infoText.set("Current process: Stacking")
            astrotrails.stacking(directory, imageList, imageName, path)
        elif selectedMode == "Video":
            if not os.path.exists(sbsPath):
                os.mkdir(sbsPath)
                infoText.set("Current process: Stacking Step by step")
                astrotrails.stackingSBS(directory, imageList, sbsPath)
                infoText.set("Current process: Video generation")
                astrotrails.timelapseVideo(path, sbsPath, fps, videoName)
            else:
                infoText.set("Current process: Video generation")
                astrotrails.timelapseVideo(path, sbsPath, fps, videoName)
        pb.destroy()
        infoLabel.destroy()
        generationText.set("Generation completed")
        w, h = self.winfo_screenwidth(), self.winfo_screenheight()
        imWid = Image.open(os.path.join(path,imageName)).size[0]
        imHgt = Image.open(os.path.join(path,imageName)).size[1]
        while imWid>(w-40) or imHgt>(h-200):
            imWid = imWid*0.98
            imHgt = imHgt*0.98
        startrailsImage = customtkinter.CTkImage(Image.open(os.path.join(path,imageName)), size=(imWid,imHgt))
        customtkinter.CTkLabel(master=self, text="", image=startrailsImage).pack()
        anotherButton = customtkinter.CTkButton(master=self, text="Generate another", font=("calibri",20), command=self.destroy)
        anotherButton.pack(side="left", padx=20, pady=20, expand=True)
        exitButton = customtkinter.CTkButton(master=self, text="Close", font=("calibri",20), command=exit)
        exitButton.pack(side="right", padx=20, pady=20, expand=True)
        self.attributes('-zoomed', True)

if __name__ == "__main__":
    App()
